/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-11-28 16:17:55.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.common.utils;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * @author: fengdesheng
 * @since: 2018/11/28
 */
public class ElectronUtils {
    private static final Logger log = LoggerFactory.getLogger(ElectronUtils.class);
    private static final String WIN_PREFIX = "cmd /c ";
    private static final String LINUX_PREFIX = "./";

    /**
     * 将URL响应内容转为PDF文件
     */
    public static boolean toPdf(String url, String path) {
        if (StringUtils.isBlank(url) || StringUtils.isBlank(path)) {
            return false;
        }

        final String script = "electron-pdf %s %s -p A3 -l true -e view-ready";
        if (isWindows()) {
            //Windows带盘符路径，去掉首个"/"字符
            if (path.indexOf(":") > 0 && path.startsWith("/")) {
                path = path.replaceFirst("/", "");
            }

            String[] cmd = new String[]{WIN_PREFIX + String.format(script, url, path)};
            return executeCommand(cmd, "GBK");
        } else {
            String cmd = LINUX_PREFIX + String.format(script, url, path);
            String[] commands = new String[]{
                    "/bin/bash",
                    "-c",
                    "export DISPLAY=':99.0'",
                    "Xvfb :99 -screen 0 1024x768x24 > /dev/null 2>&1 &",
                    "cd /usr/local/nodejs/node_global/bin", cmd
            };
            return executeCommand(commands, "UTF-8");
        }
    }

    /**
     * 执行命令
     */
    private static boolean executeCommand(String[] commands, String charsetName) {
        Process process;

        try {
            process = Runtime.getRuntime().exec(commands);
        } catch (IOException e) {
            log.error(e.getMessage());
            return false;
        }

        try (BufferedReader out = new BufferedReader(new InputStreamReader(process.getInputStream(), charsetName));
             BufferedReader err = new BufferedReader(new InputStreamReader(process.getErrorStream(), charsetName));) {

            String outStr;
            while ((outStr = out.readLine()) != null) {
                log.debug(outStr);
            }

            String errStr;
            while ((errStr = err.readLine()) != null) {
                log.debug("err:" + errStr);
            }

            int exitCode = process.waitFor();
            if (exitCode == 0) {
                log.debug("Execute Command Success.");
                return true;
            }

            log.warn("Execute Command Error. exitCode:" + exitCode);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } catch (InterruptedException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (process != null) {
                process.destroy();
            }
        }
        return false;
    }

    public static boolean isWindows() {
        String os = System.getProperty("os.name").toLowerCase();
        if (os.contains("windows") || os.startsWith("win")) {
            return true;
        }
        return false;
    }

}